@@ -2,6 +2,7 @@
 A simple python library to interact with Microsoft Graph and Office 365 API
 """
 import warnings
+import sys
 
 from .__version__ import __version__
 
@@ -11,5 +12,6 @@ from .utils import FileSystemTokenBackend
 from .message import Message
 
 
-# allow Deprecation warnings to appear
-warnings.simplefilter('always', DeprecationWarning)
+if sys.warnoptions:
+    # allow Deprecation warnings to appear
+    warnings.simplefilter('always', DeprecationWarning)
