@@ -774,7 +774,7 @@ class DriveItem(ApiComponent):
         # Find out if the server has run a Sync or Async operation
         location = response.headers.get('Location', None)
 
-        if 'monitor' in location:
+        if response.status_code == 202:
             # Async operation
             return CopyOperation(parent=self.drive, monitor_url=location)
         else:
@@ -1146,8 +1146,10 @@ class Folder(DriveItem):
             if item.is_folder and item.child_count > 0:
                 item.download_contents(to_folder=to_folder / item.name)
             elif item.is_folder and item.child_count == 0:
-                if not to_folder.exists():
-                    to_folder.mkdir()
+                # Create child folder without contents.
+                child_folder = to_folder / item.name
+                if not child_folder.exists():
+                    child_folder.mkdir()
             else:
                 item.download(to_folder)
 
@@ -1452,13 +1454,13 @@ class Drive(ApiComponent):
                                       **{self._cloud_data_key: data})
 
     def _base_get_list(self, url, limit=None, *, query=None, order_by=None,
-                       batch=None):
+                       batch=None, params = {}):
         """ Returns a collection of drive items """
 
         if limit is None or limit > self.protocol.max_top_value:
             batch = self.protocol.max_top_value
 
-        params = {'$top': batch if batch else limit}
+        params ['$top'] = batch if batch else limit
 
         if order_by:
             params['$orderby'] = order_by
@@ -1562,7 +1564,7 @@ class Drive(ApiComponent):
         return self._base_get_list(url, limit=limit, query=query,
                                    order_by=order_by, batch=batch)
 
-    def get_shared_with_me(self, limit=None, *, query=None, order_by=None,
+    def get_shared_with_me(self, limit=None, allow_external=False, *, query=None, order_by=None,
                            batch=None):
         """ Returns a collection of DriveItems shared with me
 
@@ -1573,6 +1575,8 @@ class Drive(ApiComponent):
         :type order_by: Query or str
         :param int batch: batch size, retrieves items in
          batches allowing to retrieve more items than the limit.
+        :param allow_external: includes items shared from external tenants
+        :type allow_external: bool
         :return: items in this folder
         :rtype: generator of DriveItem or Pagination
         """
@@ -1585,8 +1589,11 @@ class Drive(ApiComponent):
             # we don't know the drive_id so go to the default
             url = self.build_url(self._endpoints.get('shared_with_me_default'))
 
+        # whether to include driveitems external to tenant
+        params = {"allowexternal" : allow_external}
+
         return self._base_get_list(url, limit=limit, query=query,
-                                   order_by=order_by, batch=batch)
+                                   order_by=order_by, batch=batch, params=params)
 
     def get_item(self, item_id):
         """ Returns a DriveItem by it's Id
