@@ -14,6 +14,8 @@ ME_RESOURCE = 'me'
 USERS_RESOURCE = 'users'
 GROUPS_RESOURCE = 'groups'
 SITES_RESOURCE = 'sites'
+TEAMS_RESOURCE = 'teams'
+CHAT_RESOURCE = 'chats'
 
 NEXT_LINK_KEYWORD = '@odata.nextLink'
 
@@ -351,7 +353,8 @@ class ApiComponent:
             # user resource shorthand
             resource = resource.replace('user:', '', 1)
             return '{}/{}'.format(USERS_RESOURCE, resource)
-        elif '@' in resource and not resource.startswith(USERS_RESOURCE):
+        elif '@' in resource and True not in (resource.startswith(prefix) for prefix in
+                                              (USERS_RESOURCE, TEAMS_RESOURCE, CHAT_RESOURCE)):
             # user resource backup
             # when for example accessing a shared mailbox the
             # resource is set to the email address. we have to prefix
